% This is the demo script for the recently proposed algorithm 
% kernel adaptive dynamic texture synthesis (KDATS)

% Please cite the following paper when using this code:
% Xinge You, Weigang Guo, Shujian Yu, Kan Li, Jose C. Principe, Dacheng Tao, 
% "Kernel Learning for Dynamic Texture Synthesis," 
% IEEE Transactions on Image Processing, accepted.
% 
% Notes
% For any questions, suggestions or cooperations, please feel free to contact Shujian Yu,
% yusjlcy9011@ufl.edu
% yusj9011@gmail.com

clear;
clc;
addpath(genpath(pwd));
video=VideoReader('6.avi');
N=video.NumberOfFrames;
H=video.Height;
W=video.Width;
order=1;
f=N;
Y=zeros(f,H*W);
wuchaall=[];
for k=1:f
    mov(k).cdata=read(video,k);
    image=double(rgb2gray(mov(k).cdata));
    image=imresize(image,[H,W]);
    Y(k,:)=reshape(image,[1,H*W]);
end
Y=Y./255;
Error1=[];
Error=[];
Ymean=mean(Y,1);
Y=Y-ones(N,1)*Ymean;
% Training
for t=1
    tk=1+0.1*t;
kaf = qklms(struct('eta',0.5,'epsu',0,'kerneltype','gauss','kernelpar',3.9)); %windmill order=1
tau=f-order;
M=H*W;
for i = 1:order
    for j = 1:tau
        Z(j,(i-1)*M+1:i*M)=Y((j-1)+i,:);
    end
end
% Z=Y(1:N-1,:);
B=Y((order+1):N,:);
Y_est = zeros(N-order,M);
for i=1:(N-order)
    Y_est(i,:) = kaf.evaluate(Z(i,:),M); % make prediction
    kaf = kaf.train(Z(i,:),B(i,:),M); % train
end
% Synthesis
newobj=VideoWriter('6kadts.avi');
open(newobj);
imagenew=Z(1,:);
error=0;
for i=1:1500
%         imagenew=Z(i,:);
    newimage=kaf.evaluate(imagenew,M);
    imagenew=[imagenew(M+1:M*order),newimage];
    image=reshape(newimage+Ymean,[H,W]);
    image(image<0)=0;
    image(image>1)=1;
    writeVideo(newobj,image);
end
error=error/(N-1);
Error=[Error,error];
close(newobj);
end